-- Addon Declaration
Demon = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "FuBarPlugin-2.0")
local addon = Demon

-- Locals
local Aura = nil
local font = GameFontNormal:GetFont()
local L = AceLibrary("AceLocale-2.2"):new("Demon")
local shortDebuffCache = { }

-- Localization
L:RegisterTranslations("enUS", function() return {
	["Add"] = true,
	["Add a Debuff"] = true,
	["<Debuff in games locale | spellID>"] = true,
	["Remove"] = true,
	["Remove a Debuff"] = true,
	["Count"] = true,
	["Debuff Counter"] = true,
	["Visibility"] = true,
	["Change when you want Demon to be shown"] = true,
	["Show Focus"] = true,
	["Raid"] = true,
	["Show in a Raid"] = true,
	["Party"] = true,
	["Show in a Party"] = true,
	["Solo"] = true,
	["Show when Solo"] = true,
	["Looks"] = true,
	["Change the background of Demon"] = true,
	["Backdrop color"] = true,
	["Border color"] = true,
	
	["|cffffff00Right-Click|r to open the configuration menu"] = true,
	["|cffffff00Alt&Drag|r move the frame"] = true,
	
	["ShortSubstitutions"] =
	{
		[" [Aa][Nn][Dd] "]	= " & ",
		[" [Oo][Ff] "]		= " of ",		-- Forces shortened form "o" to go lower case
		[" [Oo][Nn] "]		= " on ",		-- Forces shortened form "o" to go lower case
		[" [Tt][Hh][Ee] "]		= "  ",		-- Removes "the" from CotE
		[" [Ii][Nn] "]		= " in ",
		["(Feral)"]			= " ",
		["(Bear)"]			= " ",
		["(Cat)"]			= " ",
	}
} end)

L:RegisterTranslations("frFR", function() return {
	["Add"] = "Ajouter",
	["Add a Debuff"] = "Ajouter un affaiblissement",
	["<Debuff in games locale | spellID>"] = "<Affaiblissement dans la langue du jeu | spellID>",
	["Remove"] = "Enlever",
	["Remove a Debuff"] = "Enlever un affaiblissement",
	["Count"] = "Compteur",
	["Debuff Counter"] = "Compteur d'affaiblissements",
	["Visibility"] = "Visibilit",
	["Change when you want Demon to be shown"] = "Afficher Demon",
	["Show Focus"] = "Afficher Focus",
	["Raid"] = "Raid",
	["Show in a Raid"] = "Afficher en raid",
	["Party"] = "Groupe",
	["Show in a Party"] = "Afficher en groupe",
	["Solo"] = "Solo",
	["Show when Solo"] = "Afficher en solo",
	["Looks"] = "Apparence",
	["Change the background of Demon"] = "Changer l'apparence de Demon",
	["Backdrop color"] = "Couleur de fond",
	["Border color"] = "Couleur de la bordure",

	["|cffffff00Right-Click|r to open the configuration menu"] = "|cffffff00Clic-droit|r pour ouvrir le menu de configuration",
	["|cffffff00Alt&Drag|r move the frame"] = "|cffffff00Alt&Glisser-dplacer|r pour dplacer la fentre",

	["ShortSubstitutions"] =
	{
		[" [Dd][Ee] "] = " de ", 
		[" [Ll][Ee] "] = " ", 
		[" [Ll][Aa] "] = " ",
		[" L' "] = " ",
	}
} end)


local optDebuffs = {
	["Thunder Clap"] = { "Thunderfury" },
	["Faerie Fire"] = { "Faerie Fire (Feral)" },
	["Sunder Armor"] = { "Expose Armor" },
	["Demoralizing Shout"] = { "Curse of Weakness", "Demoralizing Roar" },
	["Mangle"] = { "Mangle (Cat)", "Mangle (Bear)" },
}

	


-- Initialization
function addon:OnInitialize()
	local validate = {}
	self:RegisterDB("DEMONDB")
	
	-- Defaults	
self.defaults = {
	
	["Focus"] = true,
	["Raid"] = true,
	["Party"] = false,
	["Solo"] = false,
	
	["Count"] = true,
	
	["backdrop"] = {
		["r"] = 0,
		["g"] = 0,
		["b"] = 0,
		["a"] = 1,
	},
	["border"] = {
		["r"] = 0,
		["g"] = 0,
		["b"] = 0,
		["a"] = 1,
	},
}

self:RegisterDefaults('profile', self.defaults)

if not self.db.profile.Debuffs then 

	self.db.profile.Debuffs = {
		"Demoralizing Shout",
		"Thunder Clap",
		"Sunder Armor",
		"Curse of the Elements",
--		"Curse of Shadow", --doesn't exist anymore
		"Curse of Recklessness",
		"Faerie Fire",
	}
	
end

-- Options	
	self.optionsTable = {
		type = "group",
		args = {
			Add = {
				type = 'text',
				name = L['Add'],
				desc = L['Add a Debuff'],
				get = false,
				order = 1,
				set = function(value)
					if tonumber(value) then
						value = GetSpellInfo(value)
						if not value then
							return
						end
					end
					
					for _, v in ipairs(self.db.profile.Debuffs) do
						if v == value then
							return
						end
					end
					
					table.insert( self.db.profile.Debuffs, value )
					
					local k = #self.db.profile.Debuffs
					local f = self.myframe	
					
						ff = f:CreateFontString(nil,"OVERLAY")
						f.target[k] = ff
						ff:SetFont(font, 11)
						ff:SetWidth(27)
						ff:SetHeight(15)
						ff:SetText("")
						ff:SetJustifyH("CENTER")
						ff:ClearAllPoints()
						ff:SetPoint("TOPRIGHT", f.target[k-1], "TOPRIGHT", 0, -2 )		
						
						ff = f:CreateFontString(nil,"OVERLAY")
						f.focus[k] = ff
						ff:SetFont(font, 11)
						ff:SetWidth(27)
						ff:SetHeight(15)
						ff:SetText("")
						ff:SetJustifyH("CENTER")
						ff:ClearAllPoints()
						ff:SetPoint("TOPRIGHT", f.focus[k-1], "TOPRIGHT", 0, -2 )	
						
						f.target.footer:SetPoint("TOPLEFT", f.target[k], "TOPLEFT", 0, - 10 )
						f.focus.footer:SetPoint("TOPRIGHT", f.focus[k], "TOPRIGHT", 0,  -10 )	
						
						f:SetHeight( 20 + (k*10) )
						
						addon:DebuffCheck()
						
					end,
				validate = function(value)
					if tonumber(value) then
						return true
					else
						local spell = nil
						for i = 1, 10000 do
							spell = GetSpellInfo(i)
							if spell == value then
								return true
							end
						end
						return false
					end
				end,
				usage = L["<Debuff in games locale | spellID>"],
			},
			Remove = {
				type = 'text',
				name = L['Remove'],
				desc = L['Remove a Debuff'],
				order = 2,
				multiToggle = true,
				get = function() return false end,
				set = function(key)
					local f = self.myframe
					for k, value in ipairs(self.db.profile.Debuffs) do
						if value == key then
							table.remove(self.db.profile.Debuffs, k) 
						end
					end
					
					local k = #self.db.profile.Debuffs
						
					f:SetHeight( 30 + (k*10) )
					f.target[k+1]:Hide()
					f.focus[k+1]:Hide()
					
					f.target.footer:SetPoint("TOPLEFT", f.target[k], "TOPLEFT", 0, - 10 )
					f.focus.footer:SetPoint("TOPRIGHT", f.focus[k], "TOPRIGHT", -0,  -10 )	
					
					addon:DebuffCheck()
					validate[key] = nil end,
					validate = self.db.profile.Debuffs,
			},
			Count = {
				type = 'toggle',
				name = L['Count'],
				desc = L['Debuff Counter'],
				order = 3,
				get = function()
					return self.db.profile.Count
				end,
				set = function(v)
					self.db.profile.Count = v
					addon:DebuffCheck()
				end,
				map = { [false] = "Disabled", [true] = "Enabled" },
			},
			Visability = {
				name = L['Visibility'], desc=L['Change when you want Demon to be shown'], type = "group", order = 4,
				args = {
					Focus = {
						type = 'toggle',
						name = L['Show Focus'],
						desc = L['Show Focus'],
						order = 1,
						get = function()
							return self.db.profile.Focus
						end,
						set = function(v)
							self.db.profile.Focus = v
						end,
						map = { [false] = "Disabled", [true] = "Enabled" },
					},
					Raid = {
						type = 'toggle',
						name = L['Raid'],
						desc = L['Show in a Raid'],
						order = 2,
						get = function()
							return self.db.profile.Raid
						end,
						set = function(v)
							self.db.profile.Raid = v
						end,
						map = { [false] = "Disabled", [true] = "Enabled" },
					},
					Party = {
						type = 'toggle',
						name = L['Party'],
						desc = L['Show in a Party'],
						order = 3,
						get = function()
							return self.db.profile.Party
						end,
						set = function(v)
							self.db.profile.Party = v
						end,
						map = { [false] = "Disabled", [true] = "Enabled" },
					},
					Solo = {
						type = 'toggle',
						name = L['Solo'],
						desc = L['Show when Solo'],
						order = 4,
						get = function()
							return self.db.profile.Solo
						end,
						set = function(v)
							self.db.profile.Solo = v
						end,
						map = { [false] = "Disabled", [true] = "Enabled" },
					},
				},
			},
			Looks = {
				name = L['Looks'], desc=L['Change the background of Demon'], type = "group", order = 5,
				args = {
					Backdrop = {
						name = L['Backdrop color'], desc = L['Backdrop color'], type = "color", order = 1,
						hasAlpha = true,
						get = function()
							return self.db.profile.backdrop.r, self.db.profile.backdrop.g, self.db.profile.backdrop.b, self.db.profile.backdrop.a
						end,
						set = function(r, g, b, a)
							self.db.profile.backdrop.r, self.db.profile.backdrop.g, self.db.profile.backdrop.b, self.db.profile.backdrop.a = r, g, b, a
							self:SetColors()
						end,
					},
					Border = {
						name = L['Border color'], desc = L['Border color'], type = "color", order = 2,
						hasAlpha = true,
						get = function()
							return self.db.profile.border.r, self.db.profile.border.g, self.db.profile.border.b, self.db.profile.border.a
						end,
						set = function(r, g, b, a)
							self.db.profile.border.r, self.db.profile.border.g, self.db.profile.border.b, self.db.profile.border.a = r, g, b, a
							self:SetColors()
						end,
					},
				},
			},
		}, 
	}
	
	self:RegisterChatCommand({"/demon"}, self.optionsTable)
	self.OnMenuRequest = self.optionsTable
	
end

function addon:OnEnable()
	self:RegisterBucketEvent("UNIT_AURA", 0.2, "DebuffCheck")
	self:RegisterEvent("PLAYER_FOCUS_CHANGED", "DebuffCheck")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "DebuffCheck")
	if not Aura then
		Aura = AceLibrary("SpecialEvents-Aura-2.0")
	end
	self:CreateFrame()
	self:SetColors()
	self:DebuffCheck()
	self:RestorePosition()
end

-- Super Hightech Shortening device (Credits to Roartindon)
function addon:ShortenName(debuff)

	local _, numberOfSpaces = debuff:gsub(" ", "")
	
	for k,v in pairs(L["ShortSubstitutions"]) do
		debuff = debuff:gsub(k, v)
	end
	
	if numberOfSpaces == 0 then
		return debuff:sub(1, 2)
	end

	return debuff:gsub("(%a)%a*%A*", "%1")
end

-- Frame Creation
function addon:CreateFrame()
	local debuffs = self.db.profile.Debuffs
	local f = CreateFrame("Frame", "DemonFrame", UIParent)
	self.myframe = f

	f:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	})

	f:SetWidth(64)
	f:EnableMouse(true)
	f:SetMovable(true)
	f:RegisterForDrag"LeftButton"
	f:SetScript("OnDragStart", function() if IsAltKeyDown() then this:StartMoving() end end)
	f:SetScript("OnDragStop", function() this:StopMovingOrSizing() self:SavePosition() end)
	f:SetPoint("CENTER", UIParent)

	f.focus = {}
	f.target = {}
	local ff = f:CreateFontString(nil,"OVERLAY")
	f.target.header = ff
	ff:SetFont(font, 11)
	ff:SetWidth(27)
	ff:SetHeight(15)
	ff:SetText("T")
	ff:SetJustifyH("CENTER")
	ff:ClearAllPoints()
	ff:SetPoint("TOPLEFT", f, "TOPLEFT", 4, -2 )
	
	ff = f:CreateFontString(nil,"OVERLAY")
	f.focus.header = ff
	ff:SetFont(font, 11)
	ff:SetWidth(27)
	ff:SetHeight(15)
	ff:SetText("F")
	ff:SetJustifyH("CENTER")
	ff:ClearAllPoints()
	ff:SetPoint("TOPRIGHT", f, "TOPRIGHT", -4, -2 )
	
	for k, debuff in ipairs( debuffs ) do
		
		local offset = -2 - (k * 10)
	
		ff = f:CreateFontString(nil,"OVERLAY")
		f.target[k] = ff
		ff:SetFont(font, 11)
		ff:SetWidth(27)
		ff:SetHeight(15)
		ff:SetText("")
		ff:SetJustifyH("CENTER")
		ff:ClearAllPoints()
		ff:SetPoint("TOPLEFT", f, "TOPLEFT", 4, offset )
		
		ff = f:CreateFontString(nil,"OVERLAY")
		f.focus[k] = ff
		ff:SetFont(font, 11)
		ff:SetWidth(27)
		ff:SetHeight(15)
		ff:SetText("")
		ff:SetJustifyH("CENTER")
		ff:ClearAllPoints()
		ff:SetPoint("TOPRIGHT", f, "TOPRIGHT", -4, offset )		
		
		f:SetHeight( 30 + (k*10) )

	end
	
	local k = #self.db.profile.Debuffs
	
	ff = f:CreateFontString(nil,"OVERLAY")
	f.target.footer = ff
	ff:SetFont(font, 11)
	ff:SetWidth(27)
	ff:SetHeight(15)
	ff:SetText("c")
	ff:SetJustifyH("CENTER")
	ff:ClearAllPoints()
	ff:SetPoint("TOPLEFT", f, "TOPLEFT", 4, -2 - ((k+1) * 10) )
	
	ff = f:CreateFontString(nil,"OVERLAY")
	f.focus.footer = ff
	ff:SetFont(font, 11)
	ff:SetWidth(27)
	ff:SetHeight(15)
	ff:SetText("c")
	ff:SetJustifyH("CENTER")
	ff:ClearAllPoints()
	ff:SetPoint("TOPRIGHT", f, "TOPRIGHT", -4, -2 - ((k+1) * 10) )	
	
	f:Show()
end


-- Checking
function addon:DebuffCheck()
	local f = self.myframe
	local debuffs = self.db.profile.Debuffs
	if ( GetNumRaidMembers() > 0 and self.db.profile.Raid == true ) or ( GetNumPartyMembers() > 0 and self.db.profile.Party == true ) or ( self.db.profile.Solo == true ) then
		if ( UnitExists("focus") and not UnitIsPlayer("focus") ) and ( UnitExists("target") and not UnitIsPlayer("target") and self.db.profile.Focus == true) then
			f:Show()			
			local optdebuff
			for k, debuff in ipairs( debuffs ) do
				local offset = -2 - (k * 10)
				optdebuff = optDebuffs[debuff]
				self:UpdateDebuff("target", k, debuff, optdebuff)
				self:UpdateDebuff("focus", k, debuff, optdebuff)
				if self.db.profile.Count == true then
					self:UpdateDebuffCount("target")
					self:UpdateDebuffCount("focus")
					f.focus.footer:Show()
					f.target.footer:Show()
					f:SetHeight(30 + (k*10))
				else
					f.focus.footer:Hide()
					f.target.footer:Hide()
					f:SetHeight(20 + (k*10))
				end
				f:SetWidth(64)
				--Eek!
				f.target[k]:ClearAllPoints()
				f.focus[k]:ClearAllPoints()
				f.target[k]:SetPoint("TOPLEFT", f, "TOPLEFT", 4, offset )
				f.focus[k]:SetPoint("TOPRIGHT", f, "TOPRIGHT", -4, offset )
				f.focus.header:Show()
				f.target.header:Show()
				f.focus[k]:Show()
				f.target[k]:Show()
			end
			
		elseif ( UnitExists("target") and not UnitIsPlayer("target") ) then
			f:SetWidth(35)
			f:Show()
			local optdebuff
			for k, debuff in ipairs( debuffs ) do
				local offset = -2 - (k * 10)
				optdebuff = optDebuffs[debuff]
				self:UpdateDebuff("target", k, debuff, optdebuff)
				if self.db.profile.Count == true then
					self:UpdateDebuffCount("target")
					f.target.footer:Show()
					f:SetHeight(30 + (k*10))
				else
					f.target.footer:Hide()
					f:SetHeight(20 + (k*10))
				end
				-- Eek!
				f.target[k]:SetPoint("TOPLEFT", f, "TOPLEFT", 4, offset )
				f.focus.header:Hide()
				f.target.header:Show()
				f.focus.footer:Hide()
				f.focus[k]:Hide()
				f.target[k]:Show()
			end
			
		elseif ( UnitExists("focus") and not UnitIsPlayer("focus") and self.db.profile.Focus == true) then
			f:SetWidth(35)
			f:Show()
			local optdebuff
			for k, debuff in ipairs( debuffs ) do
				local offset = -2 - (k * 10)
				optdebuff = optDebuffs[debuff]
				self:UpdateDebuff("focus", k, debuff, optdebuff)
				if self.db.profile.Count == true then
					self:UpdateDebuffCount("focus")
					f.focus.footer:Show()
					f:SetHeight(30 + (k*10))
				else
					f.focus.footer:Hide()
					f:SetHeight(20 + (k*10))
				end
				-- Eek!
				f.focus[k]:SetPoint("TOPLEFT", f, "TOPLEFT", 4, offset )	
				f.focus.header:Show()
				f.target.header:Hide()
				f.focus.footer:Show()
				f.target.footer:Hide()
				f.target[k]:Hide()
				f.focus[k]:Show()
			end
			
		else		
			f:Hide()
		end
	else
		f:Hide()
	end
end

function addon:UpdateDebuff( unit, k, debuff, optdebuff )

	local shortDebuff = shortDebuffCache[debuff]
	local shortDebuffOpt = shortDebuffCache[optdebuff]
	
	if not shortDebuff then
		shortDebuff = self:ShortenName(debuff)
		shortDebuffCache[debuff] = shortDebuff
	end
		
	local text
		if UnitExists(unit) and not UnitIsPlayer(unit) then
			text = "|cffff0000"..shortDebuff.."|r"
		if Aura:UnitHasDebuff(unit, debuff) then
			text = "|cff00ff00"..shortDebuff.."|r"
		elseif optdebuff then
				for k, optd in ipairs( optdebuff ) do
				if Aura:UnitHasDebuff(unit, optd) then
				
					if not shortDebuffOpt then
						shortDebuffOpt = self:ShortenName(optd)
						shortDebuffCache[optd] = shortDebuffOpt
					end
					
					text = "|cff00ff00"..shortDebuffOpt.."|r"
					break
				end
			end
		end
	else
		text = ""
	end

	self.myframe[unit][k]:SetText( text )
end

--Counter
function addon:UpdateDebuffCount(unit)

	if self.db.profile.Count == false then
	return end
	
	local text
	local i = 1
	local count = 0


	if UnitExists(unit) and not UnitIsPlayer(unit) then
		while (i <= 40) do
			local name = UnitDebuff(unit, i)
			if (name ~= nil) then
				count = count + 1
			else
				break
			end
			i = i + 1
		end
	end
	
	if (count <= 25) then
		text = "|cff00FF00"..count.."|r"
	elseif ( count > 25 and count <= 30 ) then
		text = "|cffFFFF00"..count.."|r"
	elseif (count > 30 and count <= 35 ) then
		text = "|cffF88017"..count.."|r"
	elseif (count > 35 ) then
		text = "|cffFF0000"..count.."|r"
	else
		text = "|cffFF0000!40!|r"
	end
	
	self.myframe[unit].footer:SetText( text )
end

-- Coloring
function addon:SetColors()
	if not self.myframe then return end
	
	local backdrop = self.db.profile.backdrop
	local border = self.db.profile.border
	
	self.myframe:SetBackdropColor(backdrop.r, backdrop.g, backdrop.b, backdrop.a)
	self.myframe:SetBackdropBorderColor(border.r, border.g, border.b, border.a)
end

-- Position Saving and Restoring
function addon:SavePosition()

	local f = self.myframe

	self.db.profile.posx = f:GetLeft()
	self.db.profile.posy = f:GetTop()

end

function addon:RestorePosition()

    local f = self.myframe
	local x = self.db.profile.posx
	local y = self.db.profile.posy

	if not x or not y then return end

	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)

end

--FuBar
Demon.hasNoText = true
Demon.defaultPosition = "CENTER"
Demon.tooltipHiddenWhenEmpty = true
Demon.hasIcon = "Interface\\Icons\\Spell_Shadow_SummonVoidWalker"
Demon.defaultMinimapPosition = 285
Demon.cannotDetachTooltip = true
Demon.blizzardTooltip = true
Demon.hasNoColor = true
Demon.clickableTooltip = false
Demon.independentProfile = true
Demon.hideWithoutStandby = true

function addon:OnTooltipUpdate()
	GameTooltip:AddLine('Demon')
	GameTooltip:AddLine(' ')
	GameTooltip:AddLine(L['|cffffff00Right-Click|r to open the configuration menu'], 0.2, 1, 0.2)
	GameTooltip:AddLine(L['|cffffff00Alt&Drag|r move the frame'], 0.2, 1, 0.2)
end 
